"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserEventType = void 0;
const api_service_1 = require("./api.service");
const featureFlagResource = '/feature-flags';
const userResource = '/users';
const userEventResource = '/user-events';
const storagePath = '@growthflags/js-sdk';
const consoleLogPrefix = '@growthflags/js-sdk error:';
const isBrowser = typeof document !== 'undefined';
const isReactNative = typeof navigator !== 'undefined' && navigator.product === 'ReactNative';
const isLocalStorageSupported = isBrowser && !isReactNative;
var UserEventType;
(function (UserEventType) {
    UserEventType["FeatureViewed"] = "featureViewed";
})(UserEventType = exports.UserEventType || (exports.UserEventType = {}));
;
class FeatureFlags {
    constructor({ publicApiKey, env }) {
        this._apiKey = publicApiKey;
        this._env = env;
        this._features = {};
        this._featureOverrides = [];
        this._configs = {};
    }
    fetchFeatureFlags(user) {
        return __awaiter(this, void 0, void 0, function* () {
            if (user === null || user === void 0 ? void 0 : user.email) {
                yield this._createUser({ email: user.email, data: user.data });
            }
            if (isLocalStorageSupported) {
                const storageData = this._getFromLocalStorage();
                if (storageData) {
                    this._features = storageData.features || {};
                    this._configs = storageData.configs || {};
                }
            }
            return this._fetchFlags();
        });
    }
    isOn(featureName) {
        return Boolean(this._features[featureName]);
    }
    mergeFeatures(features) {
        let newFeatures = features;
        this._featureOverrides.forEach((featureOverride) => {
            newFeatures[featureOverride.name] = featureOverride.enabled;
        });
        return newFeatures;
    }
    setFeatures(featureOverrides) {
        this._featureOverrides = featureOverrides;
        this._features = this.mergeFeatures(this._features);
    }
    getConfig(featureName) {
        return this._configs[featureName];
    }
    trackFeatureView(featureName) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._user)
                return;
            const data = {
                userId: (_a = this._user) === null || _a === void 0 ? void 0 : _a._id,
                type: UserEventType.FeatureViewed,
                data: {
                    featureName
                }
            };
            const config = {
                headers: {
                    Authorization: 'Bearer ' + this._apiKey,
                },
            };
            try {
                yield api_service_1.default.post(`${userEventResource}`, data, config);
            }
            catch (error) {
                console.log(consoleLogPrefix, error);
            }
        });
    }
    _fetchFlags() {
        return __awaiter(this, void 0, void 0, function* () {
            const params = { env: this._env };
            if (this._user && this._user.email) {
                params.email = this._user.email;
            }
            const config = {
                headers: {
                    Authorization: 'Bearer ' + this._apiKey,
                },
            };
            try {
                const response = yield api_service_1.default.get(`${featureFlagResource}/features`, params, config);
                const features = response.features || {};
                this._features = this.mergeFeatures(features);
                this._configs = response.configs || {};
                if (isLocalStorageSupported) {
                    this._saveToLocalStorage(response);
                }
            }
            catch (error) {
                console.log(consoleLogPrefix, error);
            }
        });
    }
    _createUser(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const { email, data } = user;
            const reqData = { env: this._env, email, data };
            const config = {
                headers: {
                    Authorization: 'Bearer ' + this._apiKey,
                },
            };
            try {
                const response = yield api_service_1.default.post(`${userResource}`, reqData, config);
                this._user = response;
                if (isLocalStorageSupported) {
                    this._saveToLocalStorage({ user: response });
                }
            }
            catch (error) {
                console.log(consoleLogPrefix, error);
            }
        });
    }
    _saveToLocalStorage(data) {
        try {
            const localStorageData = this._getFromLocalStorage();
            const updatedData = Object.assign(Object.assign({}, localStorageData), data);
            const stringData = JSON.stringify(updatedData);
            localStorage.setItem(`${storagePath}`, stringData);
        }
        catch (error) {
            console.log(consoleLogPrefix, error);
        }
    }
    _getFromLocalStorage() {
        try {
            const localStorageData = localStorage.getItem(`${storagePath}`);
            return localStorageData ? JSON.parse(localStorageData) : {};
        }
        catch (error) {
            console.log(consoleLogPrefix, error);
        }
    }
}
let instance;
exports.default = {
    create: ({ publicApiKey, env }) => {
        if (instance) {
            return instance;
        }
        instance = new FeatureFlags({ publicApiKey, env });
        return instance;
    }
};
